import $ from 'jquery';

const Input = (($) => {
  /**
   * ------------------------------------------------------------------------
   * 配置信息
   * ------------------------------------------------------------------------
   */
  const NAME = 'input';
  const DATA_KEY = 'fuc.input';
  const EVENT_KEY = `.${DATA_KEY}`;
  const DATA_API_KEY = '.data-api';
  const JQUERY_NO_CONFLICT = $.fn[NAME];
  const NumberAdjust = 10000000000000000;
  const ClassName = {
    InputGroup: 'fuc-input-group',
    Input: 'fuc-input-control',
    InputErr: 'fuc-input-err',
    InputErrMsg: 'fuc-input-msg-err',
    step: {
      STEP_BTN_BOX: 'fuc-input-group-stepbox',
      STEP_BTN_UP: 'fuc-input-up',
      STEP_BTN_DN: 'fuc-input-dn',
    },
  };
  const Selector = {
    publicAttr: {
      Group: 'fuc-inputgroup-type',
      Input: 'fuc-input-type',
    },
    step: {
      STEP_GROUP_TYPE: '[fuc-inputgroup-type="step"]',
      STEP_TYPE: '[fuc-input-type="step"]',
      attr: {
        STEP: 'data-step',
        MIN: 'data-min',
        MAX: 'data-max',
      },
    },
    number: {
      NUMBER_GROUP_TYPE: '[fuc-inputgroup-type="number"]',
      NUMBER_TYPE: '[fuc-input-type="number"]',
    },
  };
  const Event = {
    KEYDOWN: `keydown${EVENT_KEY}${DATA_API_KEY}`,
    KEYUP: `keyup${EVENT_KEY}${DATA_API_KEY}`,
    CLICK: `click${EVENT_KEY}${DATA_API_KEY}`,
    BLUR: `blur${EVENT_KEY}${DATA_API_KEY}`,
  };
  const StepData = {
    name: 'step',
    selector: Selector.step,
    className: ClassName.step,
    defaultAttr: {
      step: 1,
      min: 0,
      max: 100,
    }
  };
  const NumberDaeta = {
    name: 'number',
    selector: Selector.number,
  };
  // 数字[48~57],小键盘数字[96~105],方向键[37~40],other[delate-8/46、tab-9、enter-13/108、dot-190/110]
  const isNumberKeyCode = (key) => {
    return ((key >= 48 && key <= 57) || (key >= 96 && key <= 105));
  };
  const isDotKeyCode = (key) => {
    return (key === 190 || key === 110);
  };
  const isDireKeyCode = (key) => {
    return (key >= 37 && key <= 40);
  };
  const isDelKeyCode = (key) => {
    return (key === 8 || key === 46);
  };
  const isTabKeyCode = (key) => {
    return (key === 9);
  };
  const isEnterKeyCode = (key) => {
    return (key === 13 || key === 108);
  };

  /**
   * ------------------------------------------------------------------------
   * class Input
   * ------------------------------------------------------------------------
   */
  class Input {
    constructor(element) {
      this.els = element;
      if (element.hasClass(ClassName.InputGroup)) {
        this.type = 'groupType';
        this.groupType = element.attr(Selector.publicAttr.Group) || '';
      } else if (element.hasClass(ClassName.Input)) {
        this.type = 'inputType';
        this.inputType = element.attr(Selector.publicAttr.Input) || '';
      }
      // console.log('constructor Input ', this);
    }

    // 初始化Number类型
    initNumber() {
      const $els = $(this);
      $els.on(Event.KEYDOWN, ((event) => {
        return (isNumberKeyCode(event.keyCode) || isDireKeyCode(event.keyCode) || isDelKeyCode(event.keyCode) || isTabKeyCode(event.keyCode) || isEnterKeyCode(event.keyCode));
      }));

      $els.on(Event.KEYUP, (function () { // 纠正中文
        $(this).val($(this).val().replace(/[^\d]/g, ''));
      }));
    }

    // 设置微调输入框
    initStep() {
      const {name, selector, className, defaultAttr} = StepData;
      if ($(this).attr(Selector.publicAttr.Group) !== name) return false;
      const $els = $(this).html('');
      const step = Number($els.attr(selector.attr.STEP)) || defaultAttr.step;
      const min = Number($els.attr(selector.attr.MIN)) || defaultAttr.min;
      const max = Number($els.attr(selector.attr.MAX)) || defaultAttr.max;
      const $ipt = $(`<input type="text" ${Selector.publicAttr.Input}="${name}" class="${ClassName.Input}">`);
      const $btnBox = $(`<span class="${className.STEP_BTN_BOX}"><i class="${className.STEP_BTN_UP}">+</i><i class="${className.STEP_BTN_DN}">-</i></span>`);
      $els.append($ipt).append($btnBox);

      const addStep = () => {
        let val = Number($ipt.val().replace(/[^\d|.\-+]/g, ''));
        val = (val * NumberAdjust + step * NumberAdjust) / NumberAdjust;
        if (val > max) val = max;
        return val;
      };
      const subStep = () => {
        let val = Number($ipt.val().replace(/[^\d|.\-+]/g, ''));
        val = (val * NumberAdjust - step * NumberAdjust) / NumberAdjust;
        if (val < min) val = min;
        return val;
      };

      $btnBox.find(`.${className.STEP_BTN_UP}`).on(Event.CLICK, () => {
        $ipt.val(addStep());
      });
      $btnBox.find(`.${className.STEP_BTN_DN}`).on(Event.CLICK, () => {
        $ipt.val(subStep());
      });

      $ipt.on(Event.KEYDOWN, (function(event) {
        console.log(this.value);
        const val = this.value.split('');
        val.splice(this.selectionStart, 0, event.key);
        // const res = Number(val.join('')); // 获取输入之后的数字，如果超出设置的max/min则返回false

        if (isDotKeyCode(event.keyCode)) {
          return (this.value.indexOf('.') === -1);
        }

        return (isNumberKeyCode(event.keyCode)
          // (isNumberKeyCode(event.keyCode) && res < max && res > min)
          || isDelKeyCode(event.keyCode)
          || isTabKeyCode(event.keyCode)
          || isEnterKeyCode(event.keyCode)
          || (isDireKeyCode(event.keyCode) && event.keyCode !== 38)); // [上]时返回false，纠正光标位置
      })).on(Event.KEYUP, (function (event) {
        let val = Number($ipt.val().replace(/[^\d|.\-+]/g, '')); // 纠正中文
        if (isNaN(val)) val = 0;
        if (isDotKeyCode(event.keyCode)) {
          const p = this.value.indexOf('.');
          console.log(p, this.value.length, val);
          if (p === this.value.length - 1) { // 如果是最后一位输入【.】直接return，不要Number
            val = $ipt.val();
          }
          if (val > max) val = max;
          else if (val < min) val = min;
          $(this).val(val);
          return true;
        }
        if (isNumberKeyCode(event.keyCode) || isDelKeyCode(event.keyCode) ) {
          if (val > max) val = max;
          else if (val < min) val = min;
        } else if (isDireKeyCode(event.keyCode) && event.keyCode === 38) { // 上
          val = addStep();
        } else if (isDireKeyCode(event.keyCode) && event.keyCode === 40) { // 下
          val = subStep();
        }
        $(this).val(val);
      })).on(Event.BLUR, function (event) {
        console.log(this.value);
        let val = Number($ipt.val().replace(/[^\d|.\-+]/g, '')); // 纠正中文
        if (isNaN(val)) val = 0;
        if (isNumberKeyCode(event.keyCode) || isDelKeyCode(event.keyCode) || isDotKeyCode(event.keyCode)) {
          if (val > max) val = max;
          else if (val < min) val = min;
        } else if (isDireKeyCode(event.keyCode) && event.keyCode === 38) { // 上
          val = addStep();
        } else if (isDireKeyCode(event.keyCode) && event.keyCode === 40) { // 下
          val = subStep();
        }
        $(this).val(val);
      });
    }

    // get value
    getValue() {
      const $els = $(this);
      switch ($els.attr(Selector.publicAttr.Group)) {
        case 'step':
          return $els.find(`.${ClassName.Input}${StepData.selector.STEP_TYPE}`).val();
        case 'number':
        case 'phone': {
          const $ipts = [];
          $els.find(`.${ClassName.Input}[${Selector.publicAttr.Input}="number"]`).each(function () {
            $ipts.push($(this).val());
          });
          return $ipts;
        }
        default:
          return $els.find(`.${ClassName.Input}`).val();
      }
    }

    // 设置异常状态
    setErrorStatus(msg) {
      const $els = $(this);
      if (!$els.hasClass(ClassName.InputErr)) $els.addClass(ClassName.InputErr);
      if (msg) {
        if ($els.find(`.${ClassName.InputErrMsg}`).length < 1) $els.append(`<div class="${ClassName.InputErrMsg}"></div>`);
        $els.find(`.${ClassName.InputErrMsg}`).text(msg);
      }
    }
    // 清除异常状态
    clearErrorStatus() {
      const $els = $(this);
      $els.removeClass(ClassName.InputErr);
      $els.find(`.${ClassName.InputErrMsg}`).remove();
    }

    // $.fn
    static _jQueryInterface(config, value) {
      let ipt = $(this).data(DATA_KEY);
      if (!ipt) {
        ipt = new Input(this);
        $(this).data(DATA_KEY, ipt);
      }

      if (config === 'getValue') {
        const values = {};
        this.each(function () {
          const name = $(this).attr('data-name');
          const value = ipt[config].bind(this)();

          if (name) {
            Object.assign(values, {[name]: value});
          }
        });
        return values;
      }

      return this.each((index, el) => {
        switch (config) {
        case 'initNumber':
        case 'initStep':
        case 'clearErrorStatus':
          ipt[config].bind(el)();
          break;
        case 'setErrorStatus':
          ipt[config].bind(el)(value);
          break;
        default:
          break;
        }
      });
    }
  }

  /**
   * ------------------------------------------------------------------------
   * jQuery
   * ------------------------------------------------------------------------
   */
  $.fn[NAME] = Input._jQueryInterface;
  $.fn[NAME].Constructor = Input;
  $.fn[NAME].noConflict = function () {
    $.fn[NAME] = JQUERY_NO_CONFLICT;
    return Input._jQueryInterface;
  };

  /**
   * ------------------------------------------------------------------------
   * init
   * ------------------------------------------------------------------------
   */
  $(() => {
    $(`.${ClassName.Input}${Selector.number.NUMBER_TYPE}`).input('initNumber');
    $(`.${ClassName.InputGroup}${Selector.step.STEP_GROUP_TYPE}`).input('initStep');
  });

  return Input;
})($);

export default Input;
